%
%  model = decomposeKernel(kernel,gwid,parameters)
%
%  Parameters:
%     kernel - real double-typed image of the kernel,
%              image background should be zero
%     gwid   - width of gaussian basis kernel
%     parameters - structure with parameters for finite differences.
%         .order - first or second order (1 or 2)
%         .degree - expansion degree (integer >=1)
%         .type - central (0) or forward/backward (1) differences
%
%  returns a model of the filter
%


function filtermodel = decomposeKernel(imgorig,gwid,parameters)

filtermodel.order = 1;
filtermodel.fw = 0;
filtermodel.A = 5;

if nargin == 3,
    if isfield(parameters,'order'),
        filtermodel.order = parameters.order;
    end;
    if isfield(parameters,'degree'),
        filtermodel.A = parameters.degree;
    end;
    if isfield(parameters,'type'),
        filtermodel.fw = parameters.type;
    end;
end

filtermodel.gwid = gwid;
filtermodel.realfilter = isreal(imgorig);




%% make zero padding to double sized kernel

img =zeros(size(imgorig)*2);
img(1:size(imgorig,1),1:size(imgorig,2)) = imgorig;
img = circshift(img,round(size(imgorig)/2));

g = fspecial('gaussian',size(img),gwid);
A = filtermodel.A;

%% compute all possible derivatives g^ij of the basis kernel
G = zeros(size(img,1),size(img,2),A^2);
cnt = 1;
for k = 0:(A-1),   
    gt = g;
    for j = 0:(A-1),       
        G(:,:,cnt) = gt*(-1)^(j+k);
        steerfac(cnt) = k-j;
        cnt = cnt + 1;
        gt = Zderivative(gt,-1,0,filtermodel.order,filtermodel.fw*(-1)^(j+k));         
    end;
    g = Zderivative(g,1,0,filtermodel.order,filtermodel.fw*(-1)^(k));   
end;
cnt =cnt-1;

%{
figure(1);
clf;
for k=1:cnt,
    subplot(A,A,k,'align');
    if steerfac(k) >= 0,
        imagesc(real(G( 16:48,16:48,k)));
    else
        imagesc(imag(G( 16:48,16:48,k)));
    end;
    axis off
end;
return
%}

%% compute pairwise inner prods. and projecton of kernel on basis kernels g^ij
for k= 1:cnt,
    for j = 1:cnt,
        if steerfac(k) == steerfac(j),
            Gmat(j,k) = sum(sum(G(:,:,k) .* conj(G(:,:,j))));
        else
            Gmat(j,k) = 0;
        end;
    end;
    Fvec(k) = sum(sum(conj(G(:,:,k)) .* img));
end;

%% solve for best approx. 
alpha = (Gmat+0.0*eye(cnt)) \ conj(Fvec');
filtermodel.alpha = reshape(alpha,[A A]);

%% reconstruct image
dest = zeros(size(img));
for k= 1:cnt,
    dest = dest + G(:,:,k)*alpha(k)*exp(i*steerfac(k)*pi*0.0);
end;
recon = real(dest);
figure(1); clf;
szim2 = round(size(img,1)/4);

subplot(1,2,1);
imagesc(img(szim2:(end-szim2),szim2:(end-szim2)));
title('original');
axis off
subplot(1,2,2);
imagesc(recon(szim2:(end-szim2),szim2:(end-szim2)));
title('reconstruction');
axis off

%% print approx. accuracy
fprintf('approximation accuracy = %.4f\n',sum(sum(recon .* img)) / norm(img,'fro') / norm(recon,'fro'));



